#!/bin/bash

# 取得內網網卡名稱
INTRANET=`ip addr show | grep -E 'inet (10\.100|192\.168)' | awk '{print $NF}'`
if [ -z "$INTRANET" ]; then
  echo ''
  echo '此主機沒有內網，無須使用此程式'
  echo ''
  exit 1
fi

# 檢查是否有輸入參數
if [ -z "$1" ]; then
  echo ''
  echo '使用說明：'
  echo "/opt/cpinst/jacky_script/cpanel_account_intranet_reject.sh --check [ 確認取得相關參數是否有誤 ]"
  echo "/opt/cpinst/jacky_script/cpanel_account_intranet_reject.sh --run [ 將 iptables 規則寫入 /etc/csf/csfpre.sh 並重啟防火牆 ]"
  echo "/opt/cpinst/jacky_script/cpanel_account_intranet_reject.sh --check 1227,1228 [如果需要排外特定帳號，請使用此方式，數值為UID]"
  echo "/opt/cpinst/jacky_script/cpanel_account_intranet_reject.sh --run 1227,1228 [如果需要排外特定帳號，請使用此方式，數值為UID]"
  echo ''
  exit 1
fi

# 取得 cPanel Account UID 列表
mapfile -t cpanel < <(whmapi1 --output=jsonpretty listaccts | jq -r '.data.acct[] | .uid' | sort)

# 取得 cPanel Account start & end UID
start="${cpanel[0]}"
end="${cpanel[${#cpanel[@]}-1]}"

# 找出 cPanel 帳號 start UID 至 end UID 範圍內有匹配的帳號資訊，輸出至 cpanel_check 陣列
for ((i=start; i<=end; i++)); do
  while IFS= read -r line; do
    cpanel_check+=("$line")
  done < <(grep "x:$i:" /etc/passwd | awk -F\: '{print $3}')
done

#cpanel_check=(1005 1003 1006 1008 1008 1010)

# 產生 diff_array 空陣列
diff_array=()

# 進行比對，將多出來的 Account UID 輸出至 diff_array 陣列
for element in "${cpanel_check[@]}"; do
  if [[ ! " ${cpanel[@]} " =~ " ${element} " ]]; then
    diff_array+=("$element")
  fi
done

# 增加手動排外帳號 ( Ex.2024/8/20 add )
if [ -n "$2" ]; then
  IFS=',' read -r -a new_values <<< "$2"
  diff_array+=("${new_values[@]}")
fi

if [ "$1" == "--check" ]; then

echo ""
echo "cPanel 帳號起始 UID: $start"
echo "cPanel 帳號結束 UID: $end"
echo ""

# 輸出多出來的 Account UID 與 帳號名稱
echo 'cPanel UID 範圍內多出來非 cPanel 的帳號如下:'
if [ ${#diff_array[@]} -ne 0 ]; then
  for elem in "${diff_array[@]}"; do
    grep "x:$elem:" /etc/passwd | awk -F: '{print $1, $3}'
  done
else
  echo '沒有多出來的 Account'
fi
echo ""

# 輸出 iptables 指令
echo '自動產生 iptables 的指令如下:'
if [ ${#diff_array[@]} -ne 0 ]; then
  for elem in "${diff_array[@]}"; do
    elem2=$(grep "x:$elem:" /etc/passwd | awk -F: '{print $3}')
    echo "iptables -A OUTPUT -o ${INTRANET} -m owner --uid-owner ${elem2} -d 0.0.0.0/0 -j ACCEPT"
  done
fi
echo "iptables -A OUTPUT -o ${INTRANET} -m owner --uid-owner ${start}:${end} -d 0.0.0.0/0 -j REJECT"
echo ""

exit

fi

if [ "$1" == "--run" ]; then

# 確認排外帳號列表是否有差異
######################

# 如果 /mnt/other_uid_list.txt 檔案不存在則建立 (2024/8/20 add)
ofile='/mnt/other_uid_list.txt'
if [ ! -f "$ofile" ]; then
    touch "$ofile"
fi

# 將目前 diff_array 陣列內容輸出至 /mnt/other_uid_list_new.txt 檔案
new_ofile='/mnt/other_uid_list_new.txt'
printf "%s\n" "${diff_array[@]}" > "$new_ofile"

# 比對 /mnt/other_uid_list.txt 與 /mnt/other_uid_list_new.txt 是否有所差異
if diff "$ofile" "$new_ofile" > /dev/null; then
    #echo "排外帳號列表沒有差異"
    other_check='not_update'
else
    #echo "排外帳號列表有差異"
    cp -f $new_ofile $ofile
    other_check='need_update'
fi

######################

# 確認cPanel帳號列表是否有差異
######################

# 如果 /mnt/uid_list.txt 檔案不存在則建立
file='/mnt/uid_list.txt'
if [ ! -f "$file" ]; then
    touch "$file"
fi

# 將目前 cpanel 陣列內容輸出至 /mnt/uid_list_new.txt 檔案
new_file='/mnt/uid_list_new.txt'
printf "%s\n" "${cpanel[@]}" > "$new_file"

# 比對 /mnt/uid_list.txt 與 /mnt/uid_list_new.txt 是否有所差異
if diff "$file" "$new_file" > /dev/null; then
    #echo "cPanel帳號列表沒有差異"
    account_check='not_update'
else
    #echo "cPanel帳號列表有差異"
    cp -f $new_file $file
    account_check='need_update'
fi

######################

# 如果排外帳號列表與cPanel帳號列表有差異，才會更新 /etc/csf/csfpre.sh 列表與重啟 csf 防火牆 (2024/8/20 add)
if [ "${other_check}" == 'need_update' ] || [ "${account_check}" == 'need_update' ]; then
  echo '需要更新'

  # 重建 /etc/csf/csfpre.sh 檔案
  touch /etc/csf/csfpre.sh
  chmod 700 /etc/csf/csfpre.sh
  echo '' > /etc/csf/csfpre.sh

  # 將 iptables 針對 UID 內網阻擋連外規則加入 /etc/csf/csfpre.sh 檔案內
  echo "#!/bin/bash" > /etc/csf/csfpre.sh
  echo "" >> /etc/csf/csfpre.sh
  if [ ${#diff_array[@]} -ne 0 ]; then
    for elem in "${diff_array[@]}"; do
      elem2=$(grep "x:$elem:" /etc/passwd | awk -F: '{print $3}')
      echo "iptables -A OUTPUT -o ${INTRANET} -m owner --uid-owner ${elem2} -d 0.0.0.0/0 -j ACCEPT" >> /etc/csf/csfpre.sh
    done
  fi
  echo "iptables -A OUTPUT -o ${INTRANET} -m owner --uid-owner ${start}:${end} -d 0.0.0.0/0 -j REJECT" >> /etc/csf/csfpre.sh

  # 重啟 csf 防火牆
  csf -ra

  exit

elif [ "${other_check}" == 'not_update' ] && [ "${account_check}" == 'not_update' ]; then

  #echo '不需要更新'
  exit 0

fi

fi

